/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.server.clientinterface.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;

/**
 * A startup component for the Local Clients.
 * This class starts the CORBA client used for alerting
 * purposes.  It reads data using the ClientServant from CORBA
 * calls by the i-scream client interface.  This data is then
 * used by the MonitorManager to pass to Monitors.  Monitors then
 * analyse the data and raise alerts if needed, these are passed to 
 * the Alerters, which send out alerts.  They are looked after by
 * the AlerterManager.
 *
 * @author  $Author: tdb $
 * @version $Id: ClientMain.java,v 1.24 2003/02/05 16:43:44 tdb Exp $
 */
public class ClientMain implements Component {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.24 $";
    
    /**
     * The friendly name for this component, used by
     * all related classes.
     */
    public static final String NAME = "LocalClient";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---

    /**
     * This starts the Local Client component.
     * This starts the ClientServant, the MonitorManager and
     * the AlerterManager, aswell as initialising any queues
     * and obtaining any initial configuration.
     *
     * @throws ComponentStartException if the component fails to start
     */
    public void start() throws ComponentStartException {
        // get references to key objects
        _logger = _refman.getLogger();

        _logger.write(toString(), Logger.SYSINIT, "coming up");
        
        // get a reference to the configuration proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        
        // see if these Queue's need a size limit
        try {
            int queueSizeLimit = Integer.parseInt(cp.getProperty(NAME, "Queue.SizeLimit"));
            String queueRemoveAlgorithm = cp.getProperty(NAME, "Queue.RemoveAlgorithm");
            int algorithm = StringUtils.getStringPos(queueRemoveAlgorithm, Queue.algorithms);
            if(algorithm != -1) {
                _logger.write(toString(), Logger.DEBUG, "Starting 2 Queues with size limit of "+queueSizeLimit+", using remove algorithm "+queueRemoveAlgorithm);
                // we have valid values, so lets start it.
                _alerterQueue = new Queue(queueSizeLimit, algorithm);
                _monitorQueue = new Queue(queueSizeLimit, algorithm);
            }
            else {
                _logger.write(toString(), Logger.WARNING, "Bad Queue Algorithm configuration, not known: "+queueRemoveAlgorithm);
                // just don't activate a limit
                _alerterQueue = new Queue();
                _monitorQueue = new Queue();
            }
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            // just don't activate a limit
            _alerterQueue = new Queue();
            _monitorQueue = new Queue();
        } catch (NumberFormatException e) {
            _logger.write(toString(), Logger.WARNING, "Bad Queue SizeLimit configuration: "+e);
            // just don't activate a limit
            _alerterQueue = new Queue();
            _monitorQueue = new Queue();
        }
        
        // startup monitors on these queues
        try {
            // try to get the interval, if this fails, we won't start up the monitor
            int queueMonitorInterval = Integer.parseInt(cp.getProperty(NAME, "Queue.MonitorInterval"));
            _alerterQueue.startMonitor(queueMonitorInterval*1000, _monitorQueue, NAME + " Alerter");
            _monitorQueue.startMonitor(queueMonitorInterval*1000, _monitorQueue, NAME + " Monitor");
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.WARNING, "failed to find queue monitor config, disabling. " + e);
        }        
        
        // setup the servant and connect
        _logger.write(toString(), Logger.SYSINIT, "starting servant and connecting");           
        try {
            ClientServant ref = new ClientServant(_monitorQueue);
            org.omg.CORBA.Object objRef = _refman.getRootPOA().servant_to_reference(ref);
            Client client = ClientHelper.narrow(objRef);
            
            // this name maybe shouldn't be static
            objRef = _refman.getCORBARef("iscream.ClientInterface\\.CorbaListener");
            CorbaClientListener listener = CorbaClientListenerHelper.narrow(objRef);
            
            _logger.write(toString(), Logger.SYSINIT, "connecting");
            CorbaControlHandler handler = listener.connect(client, NAME);
            handler.startData();
        }
        catch(Exception e) {
            // not sure what to do here
            // so we just log the error
            _logger.write(toString(), Logger.ERROR, "ERROR - " + e.getMessage());
        }
        
        // setup the MonitorManager
        MonitorManager monMan = MonitorManager.getInstance();
        monMan.start();
        
        // setup the AlerterManager
        AlerterManager alertMan = AlerterManager.getInstance();
        alertMan.start();
       
        _logger.write(toString(), Logger.SYSINIT, "started");
    }
    
    /**
     * Does a dependency check. Used mainly at startup to
     * see if the required dependencies (components) are up
     * and running.
     *
     * @return a boolean value, true if the depdencies are satisfied
     */
    public boolean depCheck() {
        try {
            org.omg.CORBA.Object obj;
            // first check the ConfigurationManager is alive
            obj = _refman.getCORBARef("iscream.ConfigurationManager");
            // then get some info on the CLI
            ConfigurationProxy cp = ConfigurationProxy.getInstance();
            String cli = cp.getProperty("RootFilter", "RootFilter.realtimeInterfaceName");
            // finally check the CLI is alive
            obj = _refman.getCORBARef("iscream.ClientInterface\\." + cli);
        } catch(ComponentCORBAException e) {
            System.err.println(toString() + ": Dependency Failure: "+e);
            return false;
        } catch(PropertyNotFoundException e) {
            System.err.println(toString() + ": Unable to obtain configuration: "+e);
            return false;
        }
        // dependency check suceeded
        return true;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            NAME,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger;

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();

//---STATIC ATTRIBUTES---

    /**
     * A queue for the alerter manager
     */
    public static Queue _alerterQueue;
    
    /**
     * A queue for the monitor manager
     */
    public static Queue _monitorQueue;

}
