/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.dbinterface;

//---IMPORTS---
import java.sql.*;
import java.util.*;

import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.clientinterface.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * A class containing a method to accept a String containing XML.
 * This string is then parsed and the contents are added to the
 * database.
 *
 * IMPORTANT: This method obtains the database user and database
 * connection password from a seperate configuration file. As such,
 * this class may be safely placed in a public CVS repository.
 *
 * @author  $Author: tdb $
 * @version $Id: DBInserter.java,v 1.30 2003/02/05 16:43:47 tdb Exp $
 */
public class DBInserter extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.30 $";

//---STATIC METHODS---

//---CONSTRUCTORS---

    public DBInserter(Queue queue) throws ComponentStartException {
        // set the Thread name
        setName("dbinterface.DBInserter");
        
        _queue = queue;
        //lets keep all this in the configuration
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String configName = "mySQL";
        try {
            _mySQLUser = cp.getProperty(configName, "mySQL.User");
            _mySQLPassword = cp.getProperty(configName, "mySQL.Password");
            _mySQLHost = cp.getProperty(configName, "mySQL.Host");
            _mySQLDatabase = cp.getProperty(configName, "mySQL.Database");
        } catch(PropertyNotFoundException e) {
            throw new ComponentStartException("Essential mySQL config not found: "+e);
        }
        _logger.write(toString(), Logger.SYSINIT, "created");
    }

//---PUBLIC METHODS---
    
    /**
     * Main run Thread of this class
     */
    public void run() {
        int qID = _queue.getQueue();
        // keep grabbing xml data, and sending it to the add method
        while(true) {
            XMLPacketWrapper xpw = null;
            try {
                xpw = (XMLPacketWrapper) _queue.get(qID);
            }
            catch (InvalidQueueException e) {
                _logger.write(toString(), Logger.ERROR, "Queue error: "+e);
                continue;
            }
            addToDatabase(xpw);
        }
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            this.getClass().getName(),
            REVISION);
    }
    
//---PRIVATE METHODS---

    /**
     * Inserts the data into the database
     *
     * @param xpw the XML data to be inserted
     */
    private void addToDatabase(XMLPacketWrapper xpw) {
        
        // get the XMLPacket
        XMLPacket packet = xpw.getXMLPacket();
        
        // Immediately reject non data type packets
        // without any further tests.
        String type = packet.getParam("packet.attributes.type");
        if (!type.equals("data")) {
            return;
        }
        
        // Store the essential fields as Strings.
        String ip = packet.getParam("packet.attributes.ip");
        String machine_name = packet.getParam("packet.attributes.machine_name");
        String sent_date = packet.getParam("packet.attributes.date");
        
        // Check for null essential fields.
        if (ip == null || machine_name == null || sent_date == null || ip.equals("") || machine_name.equals("") || sent_date.equals("")){
            _logger.write(toString(), Logger.DEBUG, "A packet reached DBInserter with some null essential fields - it will not be entered into the database.");
            return;
        }
        
        // get the XML String
        String xml = xpw.getXML();
        // get the received timestamp
        long receipt_date = xpw.getTimestamp();
        
        try {
            // Use the connection string for our group database.
            String connURL = "jdbc:mysql://"+_mySQLHost+"/"+_mySQLDatabase+"?user="+_mySQLUser+"&password="+_mySQLPassword;
            Connection conn = DriverManager.getConnection(connURL);

            PreparedStatement pstmt = conn.prepareStatement("INSERT INTO ipacket (ip, machine_name, sent_date, receipt_date, xml) VALUES (?, ?, ?, ?, ?);");
            // Execute the insert statement.
            pstmt.setString(1, ip);
            pstmt.setString(2, machine_name);
            pstmt.setLong(3, Long.parseLong(sent_date));
            pstmt.setLong(4, receipt_date);
            pstmt.setString(5, xml);
            pstmt.executeUpdate();
            
            // Close the statement and database connection.
            pstmt.close();
            conn.close();
        }
        catch(SQLException e) {
            _logger.write(toString(), Logger.ERROR, "SQLException - " + e.getMessage() + "\nSQLState: " + e.getSQLState() + "\nVendorError: " + e.getErrorCode());
        }
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * The SQL username
     */
    private String _mySQLUser;
    
    /**
     * The SQL password
     */
    private String _mySQLPassword;
    
    /**
     * The SQL database name
     */
    private String _mySQLDatabase;
    
    /**
     * The SQL Host
     */
    private String _mySQLHost;
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = DBInterface.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to our Queue
     */
    private Queue _queue;

//---STATIC ATTRIBUTES---

}   
