/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.rootfilter;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.clientinterface.*;

/**
 * The root filter is what all filters talk to
 * Hosts cannot talk to this implementation of a filter.
 * It provides hooks to all data interfaces for the system
 * namely the client interface and the db interface.
 * This is an i-scream component that starts the
 * RootFilter services.
 *
 * @author  $Author: tdb $
 * @version $Id: RootFilter.java,v 1.42 2003/02/05 16:43:48 tdb Exp $
 */
public class RootFilter implements Component {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.42 $";

//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---

    /**
     * This starts the Root Filter for the system
     */
    public void start() throws ComponentStartException {
        // get references to key objects
        _logger = _refman.getLogger();
        
        _logger.write(toString(), Logger.SYSINIT, "coming up");
        
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String configName = "RootFilter";
        
        // set the name of the root filter
        try {
            NAME = cp.getProperty(configName, "RootFilter.name");
        } catch (PropertyNotFoundException e) {
            NAME = null;
            _logger.write(toString(), Logger.WARNING, "RootFilter name not set: "+e);
        }
        
        // try and get the names of the ciReal and ciDB
        String realInterface, dbInterface;
        // first realtime
        try {
            realInterface = cp.getProperty(configName, "RootFilter.realtimeInterfaceName");
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            realInterface = null;
        }
        // next dbi
        try {
            dbInterface = cp.getProperty(configName, "RootFilter.dbInterfaceName");
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            dbInterface = null;
        }
        
        ClientInterface ciReal = null, ciDB = null;
        // get reference to the client interfaces - the real time one
        if (realInterface != null) {
            ciReal = ClientInterfaceHelper.narrow(_refman.getCORBARef("iscream.ClientInterface\\." + realInterface));
        }
        // get reference to the client interfaces - and the db one
        if (dbInterface != null) {
            ciDB = ClientInterfaceHelper.narrow(_refman.getCORBARef("iscream.ClientInterface\\." + dbInterface));
        }
        
        // setup a queue
        Queue queue;
        // see if this Queue needs a size limit
        try {
            int queueSizeLimit = Integer.parseInt(cp.getProperty(configName, "Queue.SizeLimit"));
            String queueRemoveAlgorithm = cp.getProperty(configName, "Queue.RemoveAlgorithm");
            int algorithm = StringUtils.getStringPos(queueRemoveAlgorithm, Queue.algorithms);
            if(algorithm != -1) {
                _logger.write(toString(), Logger.DEBUG, "Starting Queue with size limit of "+queueSizeLimit+", using remove algorithm "+queueRemoveAlgorithm);
                // we have valid values, so lets start it.
                queue = new Queue(queueSizeLimit, algorithm);
            }
            else {
                _logger.write(toString(), Logger.WARNING, "Bad Queue Algorithm configuration, not known: "+queueRemoveAlgorithm);
                // just don't activate a limit
                queue = new Queue();
            }
            
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            // just don't activate a limit
            queue = new Queue();
        } catch (NumberFormatException e) {
            _logger.write(toString(), Logger.WARNING, "Bad Queue SizeLimit configuration: "+e);
            // just don't activate a limit
            queue = new Queue();
        }
        
        // startup a monitor on this queue
        try {
            // try to get the interval, if this fails, we won't start up the monitor
            int queueMonitorInterval = Integer.parseInt(cp.getProperty(configName, "Queue.MonitorInterval"));
            String queueName = NAME + " RootFilter";
            queue.startMonitor(queueMonitorInterval*1000, queueName);
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.WARNING, "failed to find queue monitor config, disabling. " + e);
        }
        
        if (realInterface != null && dbInterface != null) {
            _logger.write(toString(), Logger.DEBUG, "hooked to interfaces - " + realInterface + " & " + dbInterface);
            CIWrapper c = new CIWrapper(ciReal, queue);
            c.start();
            c = new CIWrapper(ciDB, queue);
            c.start();
        } else if (realInterface == null) {        
            _logger.write(toString(), Logger.DEBUG, "hooked to interfaces - " + dbInterface);
            CIWrapper c = new CIWrapper(ciDB, queue);
            c.start();
        } else if (dbInterface == null) {
            _logger.write(toString(), Logger.DEBUG, "hooked to interfaces - " + realInterface);
            CIWrapper c = new CIWrapper(ciReal, queue);
            c.start();
        } else {
            _logger.write(toString(), Logger.WARNING, "not hooked to any client interfaces, this is probably not intentional!");
        }
                
        // RootFilterServant start (for inbound child filter data)
        _logger.write(toString(), Logger.DEBUG, "starting Root Filter");
        RootFilterServant filterServant = new RootFilterServant(queue);
        // bind to the naming service as a filter
        _refman.bindToOrb(filterServant, "iscream.Filter\\." + RootFilter.NAME);
        
        _logger.write(toString(), Logger.SYSINIT, "started");
        
    }
    
    /**
     * Does a dependency check. Used mainly at startup to
     * see if the required dependencies (components) are up
     * and running.
     *
     * @return a boolean value, true if the depdencies are satisfied
     */
    public boolean depCheck() {
        org.omg.CORBA.Object obj;
        
        // first we need to check the Configuration system is available
        try {
            // first check the ConfigurationManager is alive
            obj = _refman.getCORBARef("iscream.ConfigurationManager");
        } catch(ComponentCORBAException e) {
            System.err.println(toString() + ": Dependency Failure: "+e);
            return false;
        }
        
        // next we need to check which client interfaces *should*
        // be active, and check them. Note they do not have to
        // be enabled, and we should check this.
        // -- each check only forces a dependency if it's configured
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // first check the client interface
        try {
            String cli = cp.getProperty("RootFilter", "RootFilter.realtimeInterfaceName");
            obj = _refman.getCORBARef("iscream.ClientInterface\\." + cli);
        } catch(ComponentCORBAException e) {
            System.err.println(toString() + ": Dependency Failure: "+e);
            return false;
        } catch(PropertyNotFoundException e) {
            System.err.println(toString() + ": Client interface not configured and thus not enabled.");
        }
        // second check the database interface
        try {
            String dbi = cp.getProperty("RootFilter", "RootFilter.dbInterfaceName");
            obj = _refman.getCORBARef("iscream.ClientInterface\\." + dbi);
        } catch(ComponentCORBAException e) {
            System.err.println(toString() + ": Dependency Failure: "+e);
            return false;
        } catch(PropertyNotFoundException e) {
            System.err.println(toString() + ": Database interface not configured and thus not enabled.");
        }
        
        // dependency check suceeded
        return true;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            NAME,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger;

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();

//---STATIC ATTRIBUTES---

    /**
     * The friendly name for this component, used by
     * all related classes.
     * This is set from the configuration.
     */
    public static String NAME;

}            
